"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Auth = void 0;
const tslib_1 = require("tslib");
const lib_1 = require("@valapi/lib");
const AuthRequest_1 = require("./AuthRequest");
const AuthInstance_1 = require("./AuthInstance");
const cookie_1 = require("../utils/cookie");
class Auth extends AuthInstance_1.AuthInstance {
    constructor(config = {}) {
        super(config.user);
        this.request = new AuthRequest_1.AuthRequest(Object.assign(Object.assign({}, config), {
            cookie: this.cookie
        }));
        if (this.isAuthenticated) {
            this.request.headers.setAuthorization(`Bearer ${this.access_token}`);
            this.request.headers.set("X-Riot-Entitlements-JWT", this.entitlements_token);
        }
        this.request.headers.set("Cookie", this.cookie.getSetCookieStringsSync("https://auth.riotgames.com"));
        // GET https://valorant-api.com/internal/ritoclientversion["riotGamesApiInfo"]["VS_FIXEDFILEINFO"]["FileVersion"]
        this.sdk = config.sdk || "24.6.1.3774";
    }
    hasCookie(key) {
        return this.cookie.getCookiesSync("https://auth.riotgames.com").find(x => x.key === key) !== undefined;
    }
    analyzeCookie(key) {
        if (!this.hasCookie(key)) {
            throw new lib_1.ValError({
                name: "Auth_Cookie_Error",
                message: `${key} cookie not found`,
                data: this.cookie.getSetCookieStringsSync("https://auth.riotgames.com")
            });
        }
        this.request.headers.set("Cookie", this.cookie.getSetCookieStringsSync("https://auth.riotgames.com"));
    }
    analyzeResponseCookie(key, response) {
        var _a, _b;
        if (!this.hasCookie(key)) {
            const cookies = (0, cookie_1.getResponseCookies)(response);
            for (const cookie of cookies) {
                this.cookie.setCookieSync(cookie, (_b = (_a = response.config.url) !== null && _a !== void 0 ? _a : cookie.domain) !== null && _b !== void 0 ? _b : "riotgames.com");
            }
        }
        this.analyzeCookie(key);
    }
    captcha() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const response = yield this.request.post("https://authenticate.riotgames.com/api/v1/login", {
                clientId: "riot-client",
                language: "",
                platform: "windows",
                remember: false,
                riot_identity: {
                    language: "en_US",
                    state: "auth"
                },
                sdkVersion: this.sdk,
                type: "auth"
            });
            if (response.data.type === "error") {
                throw new lib_1.ValError({
                    name: "Auth_Captcha_Error",
                    message: "Unaccept captcha token request",
                    data: response
                });
            }
            return {
                sitekey: response.data.captcha.hcaptcha.key,
                rqdata: response.data.captcha.hcaptcha.data
            };
        });
    }
    authorize() {
        return this.request.post("https://auth.riotgames.com/api/v1/authorization", {
            client_id: "riot-client",
            nonce: lib_1.ValEncryption.randomString(16),
            redirect_uri: "http://localhost/redirect",
            response_type: "token id_token",
            scope: "account openid"
        });
    }
    reauthorize() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.cookie.removeAllCookiesSync();
            const uriResponse = yield this.authorize();
            this.analyzeResponseCookie("ssid", uriResponse);
            yield this.uriTokenization(uriResponse);
        });
    }
    login(data) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // # register
            const response = yield this.authorize();
            if (response.data.type === "error") {
                throw new lib_1.ValError({
                    name: "Auth_Cookie_Error",
                    message: "Cookie Register Error",
                    data: response
                });
            }
            this.analyzeResponseCookie("asid", response);
            // # login
            const loginResponse = yield this.request.put("https://authenticate.riotgames.com/api/v1/login", {
                riot_identity: {
                    captcha: `hcaptcha ${data.captcha}`,
                    language: "en_US",
                    password: data.password,
                    remember: true,
                    username: data.username
                },
                type: "auth"
            });
            if (loginResponse.data.type === "auth") {
                throw new lib_1.ValError({
                    name: "Auth_Identity_Error",
                    message: loginResponse.data.error,
                    data: loginResponse
                });
            }
            // # mfa
            if (loginResponse.data.type === "multifactor") {
                this.isMultifactor = true;
                return;
            }
            // # token
            yield this.loginTokenization(loginResponse.data.success.login_token);
        });
    }
    multifactor(loginCode) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            // # mfa
            var _a;
            const mfaResponse = yield this.request.put("https://authenticate.riotgames.com/api/v1/login", {
                multifactor: {
                    otp: `${loginCode}`,
                    rememberDevice: true
                },
                type: "multifactor"
            });
            if (mfaResponse.data.type !== "success") {
                throw new lib_1.ValError({
                    name: "Auth_Identity_Error",
                    message: (_a = mfaResponse.data.error) !== null && _a !== void 0 ? _a : "Unknown MFA response",
                    data: mfaResponse
                });
            }
            // # token
            yield this.loginTokenization(mfaResponse.data.success.login_token);
        });
    }
    loginTokenization(loginToken) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.login_token = loginToken;
            // # cookie login
            const cookieLoginResponse = yield this.request.post("https://auth.riotgames.com/api/v1/login-token", {
                authentication_type: "RiotAuth",
                code_verifier: "",
                login_token: this.login_token,
                persist_login: true // remember
            });
            this.analyzeResponseCookie("ssid", cookieLoginResponse);
            // # uri
            const uriResponse = yield this.authorize();
            yield this.uriTokenization(uriResponse);
        });
    }
    uriTokenization(uriResponse) {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            if (uriResponse.data.type !== "response") {
                throw new lib_1.ValError({
                    name: "Auth_Token_Error",
                    message: "Unknown URI response",
                    data: uriResponse
                });
            }
            this.uriParamsTokenization(uriResponse.data.response.parameters.uri);
            yield this.entitlementsTokenization();
        });
    }
    uriParamsTokenization(uri) {
        const url = new URL(uri);
        let searchParams;
        if (url.search) {
            // access_token=eyJ....
            searchParams = new URLSearchParams(url.search);
        }
        else if (url.hash) {
            // #access_token=eyJ....
            searchParams = new URLSearchParams(url.hash.substring(1, url.hash.length));
        }
        else {
            throw new lib_1.ValError({
                name: "Auth_Token_Error",
                message: "Unknown URI",
                data: uri
            });
        }
        this.access_token = searchParams.get("access_token");
        this.id_token = searchParams.get("id_token");
        this.session_state = searchParams.get("session_state");
        this.request.headers.setAuthorization(`Bearer ${this.access_token}`);
        super.getTokenInfo();
    }
    entitlementsTokenization() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const response = yield this.request.post("https://entitlements.auth.riotgames.com/api/token/v1");
            if (!response.data.entitlements_token) {
                throw new lib_1.ValError({
                    name: "Auth_Token_Error",
                    message: "Entitlement token is undefined",
                    data: response
                });
            }
            this.entitlements_token = response.data.entitlements_token;
            this.request.headers.set("X-Riot-Entitlements-JWT", this.entitlements_token);
        });
    }
    regionTokenization() {
        return tslib_1.__awaiter(this, void 0, void 0, function* () {
            const response = yield this.request.put("https://riot-geo.pas.si.riotgames.com/pas/v1/product/valorant", {
                id_token: this.id_token
            });
            return response.data.affinities.live;
        });
    }
}
exports.Auth = Auth;
